<?php

namespace Drupal\user_dashboard_bootstrap\Plugin\QueueWorker;

/**
 * @file
 * Contains Drupal\user_dashboard_bootstrap\Plugin\QueueWorker\UserDashboardCron.php.
 */

use Drupal\Core\Config\Config;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountSwitcher;
use Drupal\Core\Session\SessionManagerInterface;
use Drupal\user\PermissionHandlerInterface;
use Drupal\user\UserInterface;
use Drupal\user_dashboard_bootstrap\UserDashboardBlocks;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Session\SessionInterface;

/**
 * A queue worker that processes send email daily report on CRON run.
 *
 * @QueueWorker(
 *   id = "cron_user_dashboard_email",
 *   title = @Translation("Cron user dashboard sends emails daily"),
 *   cron = {"time" = 300}
 * )
 */
class UserDashboardCron extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;


  /**
   * The mail manager.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected MailManagerInterface $mailManager;

  /**
   * The account switcher service.
   *
   * @var \Drupal\Core\Session\AccountSwitcherInterface
   */
  protected $accountSwitcher;

  /**
   * Service user dashboar blocks.
   *
   * @var \Drupal\user_dashboard_bootstrap\UserDashboardBlocks
   */
  protected UserDashboardBlocks $userDashboardBlocks;

  /**
   * Service render.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  /**
   * Configuration service.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected Config $config;

  /**
   * Current user service.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected AccountInterface $currentUser;

  /**
   * The user storage.
   *
   * @var \Drupal\user\UserStorageInterface
   */
  protected $userStorage;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The session.
   *
   * @var \Symfony\Component\HttpFoundation\Session\SessionInterface
   */
  protected SessionInterface $session;

  /**
   * The session manager.
   *
   * @var \Drupal\Core\Session\SessionManagerInterface
   */
  protected SessionManagerInterface $sessionManager;

  /**
   * The permission handler.
   *
   * @var \Drupal\user\PermissionHandlerInterface
   */
  protected PermissionHandlerInterface $permissionHandler;

  /**
   * Constructs a new OrphanPurger instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Session\AccountSwitcher $account_switcher
   *   Switch user.
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager.
   * @param \Drupal\user_dashboard_bootstrap\UserDashboardBlocks $user_dashboard_blocks
   *   User dashboard blocks service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   Renderer service.
   * @param \Drupal\Core\Config\Config $config
   *   Config system.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Session\SessionManagerInterface $session_manager
   *   The session manager.
   * @param \Symfony\Component\HttpFoundation\Session\SessionInterface $session
   *   The session.
   * @param \Drupal\user\PermissionHandlerInterface $permission_handler
   *   The permission handler.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, AccountSwitcher $account_switcher, MailManagerInterface $mail_manager, UserDashboardBlocks $user_dashboard_blocks, RendererInterface $renderer, Config $config, AccountInterface $current_user, ModuleHandlerInterface $module_handler, SessionManagerInterface $session_manager, SessionInterface $session, PermissionHandlerInterface $permission_handler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->accountSwitcher = $account_switcher;
    $this->mailManager = $mail_manager;
    $this->userDashboardBlocks = $user_dashboard_blocks;
    $this->renderer = $renderer;
    $this->config = $config;
    $this->currentUser = $current_user;
    $this->userStorage = $entity_type_manager->getStorage('user');
    $this->moduleHandler = $module_handler;
    $this->sessionManager = $session_manager;
    $this->permissionHandler = $permission_handler;
    $this->session = $session;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('account_switcher'),
      $container->get('plugin.manager.mail'),
      $container->get('user_dashboard_bootstrap.blocks'),
      $container->get('renderer'),
      $container->get('config.factory')->get('system.site'),
      $container->get('current_user'),
      $container->get('module_handler'),
      $container->get('session_manager'),
      $container->get('session'),
      $container->get('user.permissions'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    if (!empty($data) && !empty($data['#uid'])) {
      $key = 'email_dailly';
      if (empty($data['key'])) {
        $data['key'] = $key;
      }
      if ($user = $this->userStorage->load($data['#uid'])) {
        $this->switchUser($user);
        $data['#regions'] = $this->userDashboardBlocks->getUserdashboardRegionsBlocks($data['#uid'], $data['blocks']);
        unset($data['blocks']);
        $template = $this->emailRenderer($data);
        $this->mailManager->getInstance([
          'module' => 'user_dashboard_bootstrap',
          'key' => $key,
        ])->mail($template);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function emailRenderer($context = []) {
    global $base_url;
    $id = $context['key'] ?? 'email_dailly';
    unset($context['key']);
    if (empty($context)) {
      return FALSE;
    }
    $body = $this->renderer->renderPlain($context);
    $body = str_replace(['href="/', 'src="/'], [
      'href="' . $base_url . base_path(),
      'src="' . $base_url . base_path(),
    ], $body);
    return [
      'id' => $id,
      'headers' => [
        'Content-type' => 'text/html; charset=UTF-8; format=flowed; delsp=yes',
        'Reply-to' => $context['#site_name'] . '<' . $context['#site_email'] . '>',
        'Content-Transfer-Encoding' => '8Bit',
        'MIME-Version' => '1.0',
      ],
      'to' => $context['#username'] . '<' . $context['#email'] . '>',
      'subject' => $context['#subject'] ?? $context['#site_name'],
      'body' => $body,
      'Return-Path' => $this->config->get('mail'),
    ];
  }

  /**
   * Logs out current user and logs in as pointed user.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user entity to switch to.
   *
   * @return \Drupal\user\UserInterface
   *   The previous user entity.
   *
   * @see \Drupal\Core\Session\SessionHandler::write()
   * @see \Drupal\user\Authentication\Provider\Cookie::getUserFromSession()
   */
  protected function switchUser(UserInterface $user) {
    // Regenerate the session ID to prevent against session fixation attacks.
    $currentUser = $this->currentUser;
    $this->sessionManager->regenerate();
    $this->accountSwitcher->switchTo($user);
    $this->session->set('uid', $user->id());
    return $currentUser;
  }

}
